@@ -43,10 +43,6 @@
 #include "curlx/inet_pton.h"
 #include "url.h"
 
-#if defined(DEBUGBUILD) && !defined(CURL_DISABLE_VERBOSE_STRINGS)
-#define DEBUG_AND_VERBOSE
-#endif
-
 /* for the (SOCKS) connect state machine */
 enum socks_state_t {
   SOCKS_ST_INIT,
@@ -72,7 +68,7 @@ enum socks_state_t {
   SOCKS_ST_FAILED
 };
 
-#ifdef DEBUG_AND_VERBOSE
+#if defined(DEBUGBUILD) && defined(CURLVERBOSE)
 static const char * const cf_socks_statename[] = {
   "SOCKS_INIT",
   "SOCKS4_START",
@@ -160,8 +156,7 @@ CURLcode Curl_blockread_all(struct Curl_cfilter *cf,
 }
 #endif
 
-#if defined(DEBUGBUILD) && !defined(CURL_DISABLE_VERBOSE_STRINGS)
-#define DEBUG_AND_VERBOSE
+#if defined(DEBUGBUILD) && defined(CURLVERBOSE)
 #define sxstate(x, c, d, y) socksstate(x, c, d, y, __LINE__)
 #else
 #define sxstate(x, c, d, y) socksstate(x, c, d, y)
@@ -172,24 +167,26 @@ static void socksstate(struct socks_state *sx,
                        struct Curl_cfilter *cf,
                        struct Curl_easy *data,
                        enum socks_state_t state
-#ifdef DEBUG_AND_VERBOSE
+#if defined(DEBUGBUILD) && defined(CURLVERBOSE)
                        , int lineno
 #endif
 )
 {
   enum socks_state_t oldstate = sx->state;
-  (void)cf;
-  (void)data;
+
   if(oldstate == state)
     /* do not bother when the new state is the same as the old state */
     return;
 
   sx->state = state;
 
-#ifdef DEBUG_AND_VERBOSE
+#if defined(DEBUGBUILD) && defined(CURLVERBOSE)
   CURL_TRC_CF(data, cf, "[%s] -> [%s] (line %d)",
               cf_socks_statename[oldstate],
               cf_socks_statename[sx->state], lineno);
+#else
+  (void)cf;
+  (void)data;
 #endif
 }
 
@@ -1288,7 +1285,7 @@ static CURLcode socks_proxy_cf_connect(struct Curl_cfilter *cf,
   else if(sx->state != SOCKS_ST_SUCCESS)
     goto out;
 
-#ifndef CURL_DISABLE_VERBOSE_STRINGS
+#ifdef CURLVERBOSE
   if(Curl_trc_is_verbose(data)) {
     struct ip_quadruple ipquad;
     bool is_ipv6;
